"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AINativeBrowserContribution = void 0;
const tslib_1 = require("tslib");
const di_1 = require("@opensumi/di");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const command_1 = require("@opensumi/ide-core-browser/lib/ai-native/command");
const ai_native_1 = require("@opensumi/ide-core-browser/lib/contextkey/ai-native");
const constants_1 = require("@opensumi/ide-core-browser/lib/layout/constants");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const ide_core_common_2 = require("@opensumi/ide-core-common");
const ide_design_1 = require("@opensumi/ide-design");
const browser_1 = require("@opensumi/ide-editor/lib/browser");
const ide_main_layout_1 = require("@opensumi/ide-main-layout");
const ide_preferences_1 = require("@opensumi/ide-preferences");
const editorExtensions_1 = require("@opensumi/monaco-editor-core/esm/vs/editor/browser/editorExtensions");
const common_1 = require("../common");
const ai_editor_contribution_1 = require("./ai-editor.contribution");
const ai_native_service_1 = require("./ai-native.service");
const chat_proxy_service_1 = require("./chat/chat-proxy.service");
const chat_view_1 = require("./chat/chat.view");
const completeProvider_1 = require("./inline-completions/completeProvider");
const ai_completions_service_1 = require("./inline-completions/service/ai-completions.service");
const tabbar_view_1 = require("./layout/tabbar.view");
const avatar_view_1 = require("./layout/view/avatar/avatar.view");
const light_bulb_widget_1 = require("./light-bulb-widget");
const run_toolbar_1 = require("./run/toolbar/run-toolbar");
const types_1 = require("./types");
let AINativeBrowserContribution = class AINativeBrowserContribution {
    constructor() {
        this.registerFeature();
    }
    initialize() {
        this.aiNativeConfigService.enableCapabilities();
        const { supportsChatAssistant } = this.aiNativeConfigService.capabilities;
        if (supportsChatAssistant) {
            ide_core_browser_1.ComponentRegistryImpl.addLayoutModule(this.appConfig.layoutConfig, common_1.AI_CHAT_VIEW_ID, common_1.AI_CHAT_CONTAINER_ID);
            ide_core_browser_1.ComponentRegistryImpl.addLayoutModule(this.appConfig.layoutConfig, ide_design_1.DESIGN_MENU_BAR_RIGHT, common_1.AI_CHAT_LOGO_AVATAR_ID);
            this.chatProxyService.registerDefaultAgent();
        }
    }
    registerEditorExtensionContribution(register) {
        const { supportsInlineChat } = this.aiNativeConfigService.capabilities;
        if (supportsInlineChat) {
            register(light_bulb_widget_1.OpenSumiLightBulbWidget.ID, light_bulb_widget_1.OpenSumiLightBulbWidget, editorExtensions_1.EditorContributionInstantiation.Lazy);
        }
    }
    onDidStart() {
        (0, ide_core_common_2.runWhenIdle)(() => {
            const prefChatVisibleType = this.preferenceService.getValid(ide_core_browser_1.AINativeSettingSectionsId.CHAT_VISIBLE_TYPE);
            if (prefChatVisibleType === 'always') {
                this.commandService.executeCommand(command_1.AI_CHAT_VISIBLE.id, true);
            }
            else if (prefChatVisibleType === 'never') {
                this.commandService.executeCommand(command_1.AI_CHAT_VISIBLE.id, false);
            }
        });
    }
    registerFeature() {
        this.contributions.getContributions().forEach((contribution) => {
            if (contribution.registerInlineChatFeature) {
                contribution.registerInlineChatFeature(this.inlineChatFeatureRegistry);
            }
            if (contribution.registerChatFeature) {
                contribution.registerChatFeature(this.chatFeatureRegistry);
            }
            if (contribution.registerResolveConflictFeature) {
                contribution.registerResolveConflictFeature(this.resolveConflictRegistry);
            }
            if (contribution.registerRenameProvider) {
                contribution.registerRenameProvider(this.renameCandidatesProviderRegistry);
            }
            if (contribution.registerChatRender) {
                contribution.registerChatRender(this.chatRenderRegistry);
            }
            if (contribution.registerTerminalProvider) {
                contribution.registerTerminalProvider(this.terminalProviderRegistry);
            }
        });
    }
    registerSetting(registry) {
        registry.registerSettingGroup({
            id: ide_core_browser_1.AI_NATIVE_SETTING_GROUP_ID,
            title: ide_core_common_2.AI_NATIVE_SETTING_GROUP_TITLE,
            iconClass: (0, ide_core_browser_1.getIcon)('magic-wand'),
        });
        registry.registerSettingSection(ide_core_browser_1.AI_NATIVE_SETTING_GROUP_ID, {
            title: (0, ide_core_browser_1.localize)('preference.ai.native.chat.title'),
            preferences: [
                {
                    id: ide_core_browser_1.AINativeSettingSectionsId.CHAT_VISIBLE_TYPE,
                    localized: 'preference.ai.native.chat.visible.type',
                },
            ],
        });
        registry.registerSettingSection(ide_core_browser_1.AI_NATIVE_SETTING_GROUP_ID, {
            title: (0, ide_core_browser_1.localize)('preference.ai.native.interface.quick.title'),
            preferences: [
                {
                    id: ide_core_browser_1.AINativeSettingSectionsId.INTERFACE_QUICK_NAVIGATION_ENABLED,
                    localized: 'preference.ai.native.interface.quick.navigation',
                },
            ],
        });
        if (this.aiNativeConfigService.capabilities.supportsInlineChat) {
            registry.registerSettingSection(ide_core_browser_1.AI_NATIVE_SETTING_GROUP_ID, {
                title: (0, ide_core_browser_1.localize)('preference.ai.native.inlineChat.title'),
                preferences: [
                    {
                        id: ide_core_browser_1.AINativeSettingSectionsId.INLINE_CHAT_AUTO_VISIBLE,
                        localized: 'preference.ai.native.inlineChat.auto.visible',
                    },
                    {
                        id: ide_core_browser_1.AINativeSettingSectionsId.INLINE_CHAT_CODE_ACTION_ENABLED,
                        localized: 'preference.ai.native.inlineChat.codeAction.enabled',
                    },
                ],
            });
        }
    }
    registerEditorFeature(registry) {
        registry.registerEditorFeatureContribution({
            contribute: (editor) => {
                const aiEditorContribution = this.injector.get(ai_editor_contribution_1.AIEditorContribution, [editor]);
                return aiEditorContribution.contribute(editor);
            },
        });
    }
    registerCommands(commands) {
        commands.registerCommand(command_1.AI_INLINE_CHAT_VISIBLE, {
            execute: (value) => {
                this.aiNativeService.launchInlineChatVisible(value);
            },
        });
        commands.registerCommand(command_1.AI_INLINE_COMPLETION_REPORTER, {
            execute: (relationId, sessionId, accept) => {
                this.aiCompletionsService.report({ sessionId, accept, relationId });
            },
        });
        commands.registerCommand(command_1.AI_CHAT_VISIBLE, {
            execute: (visible) => {
                this.layoutService.toggleSlot(common_1.AI_CHAT_VIEW_ID, (0, ide_core_common_2.isUndefined)(visible) ? true : visible);
            },
        });
        commands.registerCommand(command_1.AI_INLINE_COMPLETION_VISIBLE, {
            execute: async (visible) => {
                if (!visible) {
                    await this.commandService.executeCommand('editor.action.inlineSuggest.hide');
                    this.aiCompletionsService.hideStatusBarItem();
                    this.aiInlineCompletionsProvider.resetContextKey();
                    this.aiInlineCompletionsProvider.cancelRequest();
                    this.aiCompletionsService.setVisibleCompletion(false);
                }
            },
        });
    }
    registerRenderer(registry) {
        registry.registerSlotRenderer(common_1.AI_CHAT_VIEW_ID, tabbar_view_1.AIChatTabRenderer);
        if (this.designLayoutConfig.useMergeRightWithLeftPanel) {
            registry.registerSlotRenderer(ide_core_browser_1.SlotLocation.left, tabbar_view_1.AILeftTabRenderer);
            registry.registerSlotRenderer(ide_core_browser_1.SlotLocation.right, tabbar_view_1.AIRightTabRenderer);
        }
    }
    registerComponent(registry) {
        registry.register(common_1.AI_CHAT_CONTAINER_ID, [], {
            component: chat_view_1.AIChatView,
            containerId: common_1.AI_CHAT_CONTAINER_ID,
        });
        registry.register(common_1.AI_MENU_BAR_DEBUG_TOOLBAR, {
            id: common_1.AI_MENU_BAR_DEBUG_TOOLBAR,
            component: run_toolbar_1.AIRunToolbar,
        });
        registry.register(common_1.AI_CHAT_LOGO_AVATAR_ID, {
            id: common_1.AI_CHAT_LOGO_AVATAR_ID,
            component: avatar_view_1.AIChatLogoAvatar,
        });
    }
    registerKeybindings(keybindings) {
        if (this.aiNativeConfigService.capabilities.supportsInlineChat) {
            keybindings.registerKeybinding({
                command: command_1.AI_INLINE_CHAT_VISIBLE.id,
                keybinding: 'ctrlcmd+i',
                when: 'editorTextFocus',
                args: true,
                priority: 0,
            }, ide_core_browser_1.KeybindingScope.USER);
            keybindings.registerKeybinding({
                command: command_1.AI_INLINE_CHAT_VISIBLE.id,
                keybinding: 'esc',
                args: false,
                when: `editorFocus && ${ai_native_1.InlineChatIsVisible.raw}`,
            });
        }
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(di_1.INJECTOR_TOKEN),
    tslib_1.__metadata("design:type", di_1.Injector)
], AINativeBrowserContribution.prototype, "injector", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.AppConfig),
    tslib_1.__metadata("design:type", Object)
], AINativeBrowserContribution.prototype, "appConfig", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(types_1.AINativeCoreContribution),
    tslib_1.__metadata("design:type", Object)
], AINativeBrowserContribution.prototype, "contributions", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_2.InlineChatFeatureRegistryToken),
    tslib_1.__metadata("design:type", Object)
], AINativeBrowserContribution.prototype, "inlineChatFeatureRegistry", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_2.ChatFeatureRegistryToken),
    tslib_1.__metadata("design:type", Object)
], AINativeBrowserContribution.prototype, "chatFeatureRegistry", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_2.ChatRenderRegistryToken),
    tslib_1.__metadata("design:type", Object)
], AINativeBrowserContribution.prototype, "chatRenderRegistry", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_2.ResolveConflictRegistryToken),
    tslib_1.__metadata("design:type", Object)
], AINativeBrowserContribution.prototype, "resolveConflictRegistry", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_2.RenameCandidatesProviderRegistryToken),
    tslib_1.__metadata("design:type", Object)
], AINativeBrowserContribution.prototype, "renameCandidatesProviderRegistry", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.TerminalRegistryToken),
    tslib_1.__metadata("design:type", Object)
], AINativeBrowserContribution.prototype, "terminalProviderRegistry", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ai_native_service_1.AINativeService),
    tslib_1.__metadata("design:type", ai_native_service_1.AINativeService)
], AINativeBrowserContribution.prototype, "aiNativeService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.AINativeConfigService),
    tslib_1.__metadata("design:type", ide_core_browser_1.AINativeConfigService)
], AINativeBrowserContribution.prototype, "aiNativeConfigService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(constants_1.DesignLayoutConfig),
    tslib_1.__metadata("design:type", constants_1.DesignLayoutConfig)
], AINativeBrowserContribution.prototype, "designLayoutConfig", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ai_completions_service_1.AICompletionsService),
    tslib_1.__metadata("design:type", ai_completions_service_1.AICompletionsService)
], AINativeBrowserContribution.prototype, "aiCompletionsService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(completeProvider_1.AIInlineCompletionsProvider),
    tslib_1.__metadata("design:type", completeProvider_1.AIInlineCompletionsProvider)
], AINativeBrowserContribution.prototype, "aiInlineCompletionsProvider", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_2.CommandService),
    tslib_1.__metadata("design:type", Object)
], AINativeBrowserContribution.prototype, "commandService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.PreferenceService),
    tslib_1.__metadata("design:type", Object)
], AINativeBrowserContribution.prototype, "preferenceService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_main_layout_1.IMainLayoutService),
    tslib_1.__metadata("design:type", Object)
], AINativeBrowserContribution.prototype, "layoutService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(common_1.ChatProxyServiceToken),
    tslib_1.__metadata("design:type", chat_proxy_service_1.ChatProxyService)
], AINativeBrowserContribution.prototype, "chatProxyService", void 0);
AINativeBrowserContribution = tslib_1.__decorate([
    (0, ide_core_browser_1.Domain)(ide_core_browser_1.ClientAppContribution, browser_1.BrowserEditorContribution, ide_core_browser_1.CommandContribution, ide_preferences_1.SettingContribution, ide_core_browser_1.KeybindingContribution, ide_core_browser_1.ComponentContribution, ide_core_browser_1.SlotRendererContribution, ide_core_browser_1.MonacoContribution),
    tslib_1.__metadata("design:paramtypes", [])
], AINativeBrowserContribution);
exports.AINativeBrowserContribution = AINativeBrowserContribution;
//# sourceMappingURL=ai-core.contribution.js.map