"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PS1TerminalService = void 0;
const tslib_1 = require("tslib");
const dom_align_1 = tslib_1.__importDefault(require("dom-align"));
const react_1 = tslib_1.__importDefault(require("react"));
const client_1 = require("react-dom/client");
const di_1 = require("@opensumi/di");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const ide_terminal_next_1 = require("@opensumi/ide-terminal-next");
const stream_1 = require("@opensumi/ide-utils/lib/stream");
const terminal_command_suggest_controller_1 = require("./component/terminal-command-suggest-controller");
const terminal_feature_registry_1 = require("./terminal.feature.registry");
// 基于 PS1 Hack 的终端 AI 能力集成
var IstermOscPt;
(function (IstermOscPt) {
    IstermOscPt["PromptStarted"] = "PS";
    IstermOscPt["PromptEnded"] = "PE";
    IstermOscPt["CurrentWorkingDirectory"] = "CWD";
})(IstermOscPt || (IstermOscPt = {}));
let PS1TerminalService = class PS1TerminalService extends ide_core_common_1.Disposable {
    constructor() {
        super(...arguments);
        this.cancelToken = new ide_core_common_1.CancellationTokenSource();
    }
    active() {
        this.initContainer();
        this.disposables.push(this.terminalController.onDidOpenTerminal(({ id }) => this.listenTerminalEvent(id)));
    }
    listenTerminalEvent(clientId) {
        const client = this.terminalController.clients.get(clientId);
        if (client) {
            setTimeout(() => {
                this.listenPromptState(client.term);
            }, 0);
        }
    }
    initContainer() {
        this.popupContainer = document.createElement('div');
        document.body.appendChild(this.popupContainer);
    }
    listenPromptState(xterm) {
        xterm.parser.registerOscHandler(6973, (data) => {
            const argsIndex = data.indexOf(';');
            const sequence = argsIndex === -1 ? data : data.substring(0, argsIndex);
            switch (sequence) {
                case IstermOscPt.PromptEnded:
                    this.handlePromptEnd(xterm);
                    break;
                default:
                    return false;
            }
            // 不要阻止事件传递
            return false;
        });
    }
    handlePromptEnd(xterm) {
        if (this.promptEndMarker) {
            this.promptEndMarker.dispose();
        }
        if (this.promptEndDecoration) {
            this.promptEndDecoration.dispose();
        }
        this.promptEndMarker = xterm.registerMarker(0);
        const xOffset2 = xterm.buffer.active.cursorX;
        if (this.onDataDisposable) {
            this.onDataDisposable.dispose();
        }
        const aiHinitDecoration = xterm.registerDecoration({
            marker: this.promptEndMarker,
            width: xterm.cols,
            height: 1,
            x: xOffset2 + 2,
            layer: 'top',
        });
        aiHinitDecoration === null || aiHinitDecoration === void 0 ? void 0 : aiHinitDecoration.onRender((element) => {
            element.innerText = (0, ide_core_browser_1.localize)('terminal.ai.inputSharpToGetHint');
            element.style.opacity = '0.3';
        });
        this.onDataDisposable = xterm.onData((e) => {
            var _a;
            // 获取当前活动缓冲区
            const buffer = xterm.buffer.active;
            // 获取光标位置
            const cursorX = buffer.cursorX;
            const cursorY = buffer.cursorY;
            // 用户有输入时，就隐藏掉提示框
            aiHinitDecoration === null || aiHinitDecoration === void 0 ? void 0 : aiHinitDecoration.dispose();
            if (e === '#' &&
                cursorY + buffer.baseY === ((_a = this.promptEndMarker) === null || _a === void 0 ? void 0 : _a.line) &&
                (cursorX === xOffset2 || cursorX === xOffset2 + 1)) {
                this.showAICommandPopup(xterm, xOffset2);
            }
        });
    }
    /**
     * 展示 AI 提示命令的弹框
     * @param xterm Xterm 实例
     * @param xOffset2 传入的 X 偏移坐标
     * @returns
     */
    showAICommandPopup(xterm, xOffset2) {
        var _a, _b, _c;
        if (!this.promptEndMarker) {
            return;
        }
        this.promptEndDecoration = xterm.registerDecoration({
            marker: this.promptEndMarker,
            width: 1,
            height: 1,
            backgroundColor: '#2472C8',
            x: xOffset2,
            layer: 'top',
        });
        // HACK: 这里拿去 TerminalConnection 的方式很 Hack，看看有没有更好的办法？
        // @ts-ignore
        const connectionWrapper = xterm._addonManager._addons.find((addon) => { var _a; return !!((_a = addon === null || addon === void 0 ? void 0 : addon.instance) === null || _a === void 0 ? void 0 : _a.connection); });
        const connection = (_a = connectionWrapper === null || connectionWrapper === void 0 ? void 0 : connectionWrapper.instance) === null || _a === void 0 ? void 0 : _a.connection;
        if (!connection) {
            // eslint-disable-next-line no-console
            console.error((0, ide_core_browser_1.localize)('terminal.ai.cannotGetTerminalConnection'));
            return;
        }
        (_b = this.promptEndDecoration) === null || _b === void 0 ? void 0 : _b.onRender((element) => {
            const domWidth = element.clientWidth;
            if (this.promptEndReactRoot) {
                this.promptEndReactRoot.unmount();
            }
            this.promptEndReactRoot = (0, client_1.createRoot)(this.popupContainer);
            // 直接在 Decoration DOM 上渲染 input 输入框的话，input 的事件会被 xterm 影响导致无法聚焦
            // 因此采用外层 DOM + DOMAlign + 生命周期绑定 的方式来渲染输入框和 AI 提示框
            this.promptEndReactRoot.render(react_1.default.createElement(terminal_command_suggest_controller_1.AITerminalPrompt, { onEscTriggered: () => {
                    setTimeout(() => {
                        var _a;
                        connection.sendData('\b');
                        (_a = this.promptEndDecoration) === null || _a === void 0 ? void 0 : _a.dispose();
                    }, 0);
                }, onSuggestionClick: (suggestion) => {
                    setTimeout(() => {
                        var _a;
                        connection.sendData('\b');
                        connection.sendData(suggestion);
                        (_a = this.promptEndDecoration) === null || _a === void 0 ? void 0 : _a.dispose();
                    }, 0);
                }, getAICommandSuggestions: this.getAICommandSuggestions.bind(this), cancelAIRequst: this.stopAIStreamRequest.bind(this) }));
            (0, dom_align_1.default)(this.popupContainer, element, {
                points: ['bl', 'bl'],
                offset: [-domWidth - 6, 0],
                targetOffset: [0, 0],
                overflow: { adjustX: false, adjustY: false },
            });
        });
        // AI 输入自然语言命令的输入框被取消，需要后续处理的逻辑
        (_c = this.promptEndDecoration) === null || _c === void 0 ? void 0 : _c.onDispose(() => {
            this.stopAIStreamRequest();
            // 取消 AI 悬浮框的 React 渲染
            if (this.promptEndReactRoot) {
                this.promptEndReactRoot.unmount();
            }
            // 聚焦焦点到 Xterm 上
            xterm.focus();
        });
    }
    async stopAIStreamRequest() {
        this.cancelToken.cancel();
        this.cancelToken = new ide_core_common_1.CancellationTokenSource();
    }
    // 获取 AI 命令建议
    async getAICommandSuggestions(commandDescription, doneCallback, _thinkingCallback, suggestionCallback) {
        await this.stopAIStreamRequest();
        const reportRelationId = this.aiReporter.start('terminalAICommand', { message: commandDescription });
        const terminalReadableStream = await this.terminalFeatureRegistry.readableCommandSuggestions(commandDescription, this.cancelToken.token);
        const aiCommandSuggestions = [];
        (0, stream_1.listenReadable)(terminalReadableStream, {
            onData: (data) => {
                aiCommandSuggestions.push(data);
                suggestionCallback(aiCommandSuggestions);
            },
            onEnd: () => {
                doneCallback();
                this.aiReporter.end(reportRelationId, { message: commandDescription, success: true });
            },
        });
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_terminal_next_1.ITerminalController),
    tslib_1.__metadata("design:type", Object)
], PS1TerminalService.prototype, "terminalController", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.TerminalRegistryToken),
    tslib_1.__metadata("design:type", terminal_feature_registry_1.TerminalFeatureRegistry)
], PS1TerminalService.prototype, "terminalFeatureRegistry", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.IAIReporter),
    tslib_1.__metadata("design:type", Object)
], PS1TerminalService.prototype, "aiReporter", void 0);
PS1TerminalService = tslib_1.__decorate([
    (0, di_1.Injectable)()
], PS1TerminalService);
exports.PS1TerminalService = PS1TerminalService;
//# sourceMappingURL=ps1-terminal.service.js.map