"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TerminalFeatureRegistry = void 0;
const tslib_1 = require("tslib");
const di_1 = require("@opensumi/di");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const stream_1 = require("@opensumi/ide-utils/lib/stream");
const types_1 = require("../types");
let TerminalFeatureRegistry = class TerminalFeatureRegistry extends ide_core_common_1.Disposable {
    constructor() {
        super(...arguments);
        this.providerMap = new Set();
    }
    hasProvider() {
        return this.providerMap.size > 0;
    }
    registerCommandSuggestionsProvider(provider) {
        this.providerMap.add(provider);
    }
    async readableCommandSuggestions(message, token) {
        var _a, e_1, _b, _c;
        const providers = Array.from(this.providerMap);
        const collectStream = [];
        const collectDesc = [];
        try {
            for (var _d = true, providers_1 = tslib_1.__asyncValues(providers), providers_1_1; providers_1_1 = await providers_1.next(), _a = providers_1_1.done, !_a;) {
                _c = providers_1_1.value;
                _d = false;
                try {
                    const provider = _c;
                    const result = await provider(message, token);
                    if ((0, stream_1.isReadableStream)(result)) {
                        collectStream.push(result);
                        continue;
                    }
                    collectDesc.push(...result);
                }
                finally {
                    _d = true;
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (!_d && !_a && (_b = providers_1.return)) await _b.call(providers_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        const terminalReadableStream = types_1.TerminalSuggestionReadableStream.create();
        collectDesc.forEach(terminalReadableStream.emitData.bind(terminalReadableStream));
        // 如果没有 stream 的 provider，则直接结束
        if (collectStream.length === 0) {
            queueMicrotask(() => {
                terminalReadableStream.end();
            });
        }
        else {
            (0, stream_1.listenGroupReadable)(collectStream, {
                onData: terminalReadableStream.emitData.bind(terminalReadableStream),
                onEnd: terminalReadableStream.end.bind(terminalReadableStream),
            });
        }
        return terminalReadableStream;
    }
};
TerminalFeatureRegistry = tslib_1.__decorate([
    (0, di_1.Injectable)()
], TerminalFeatureRegistry);
exports.TerminalFeatureRegistry = TerminalFeatureRegistry;
//# sourceMappingURL=terminal.feature.registry.js.map