"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChatAgentViewService = void 0;
const tslib_1 = require("tslib");
const di_1 = require("@opensumi/di");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const common_1 = require("../../common");
const chat_proxy_service_1 = require("./chat-proxy.service");
let ChatAgentViewService = class ChatAgentViewService {
    constructor() {
        this.componentsMap = new Map();
        this.componentsDeferredMap = new Map();
    }
    registerChatComponent(config) {
        this.componentsMap.set(config.id, config);
        if (this.componentsDeferredMap.has(config.id)) {
            this.componentsDeferredMap.get(config.id).resolve(config);
            this.componentsDeferredMap.delete(config.id);
        }
        return {
            dispose: () => {
                if (this.componentsMap.get(config.id) === config) {
                    this.componentsMap.delete(config.id);
                    this.componentsDeferredMap.delete(config.id);
                }
            },
        };
    }
    getChatComponent(id) {
        if (this.componentsMap.has(id)) {
            return this.componentsMap.get(id);
        }
        if (!this.componentsDeferredMap.has(id)) {
            this.componentsDeferredMap.set(id, new ide_core_common_1.Deferred());
        }
        return null;
    }
    getChatComponentDeferred(id) {
        return this.componentsDeferredMap.get(id) || null;
    }
    getRenderAgents() {
        return this.chatAgentService.getAgents().filter((agent) => agent.id !== chat_proxy_service_1.ChatProxyService.AGENT_ID);
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(common_1.IChatAgentService),
    tslib_1.__metadata("design:type", Object)
], ChatAgentViewService.prototype, "chatAgentService", void 0);
ChatAgentViewService = tslib_1.__decorate([
    (0, di_1.Injectable)()
], ChatAgentViewService);
exports.ChatAgentViewService = ChatAgentViewService;
//# sourceMappingURL=chat-agent.view.service.js.map