"use strict";
var ChatProxyService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChatProxyService = void 0;
const tslib_1 = require("tslib");
const di_1 = require("@opensumi/di");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const command_service_1 = require("@opensumi/ide-editor/lib/browser/monaco-contrib/command/command.service");
const stream_1 = require("@opensumi/ide-utils/lib/stream");
const common_1 = require("../../common");
const chat_api_service_1 = require("./chat.api.service");
const chat_feature_registry_1 = require("./chat.feature.registry");
/**
 * @internal
 */
let ChatProxyService = ChatProxyService_1 = class ChatProxyService extends ide_core_common_1.Disposable {
    constructor() {
        super(...arguments);
        this.chatDeferred = new ide_core_common_1.Deferred();
    }
    registerDefaultAgent() {
        this.addDispose(this.chatAgentService.registerAgent({
            id: ChatProxyService_1.AGENT_ID,
            metadata: {},
            invoke: async (request, progress, _history, token) => {
                this.chatDeferred = new ide_core_common_1.Deferred();
                const { message, command } = request;
                let prompt = message;
                if (command) {
                    const commandHandler = this.chatFeatureRegistry.getSlashCommandHandler(command);
                    if (commandHandler && commandHandler.providerPrompt) {
                        const editor = this.monacoCommandRegistry.getActiveCodeEditor();
                        const slashCommandPrompt = await commandHandler.providerPrompt(message, editor);
                        prompt = slashCommandPrompt;
                    }
                }
                const stream = await this.aiBackService.requestStream(prompt, {
                    requestId: request.requestId,
                    history: this.aiChatService.getHistoryMessages(),
                }, token);
                (0, stream_1.listenReadable)(stream, {
                    onData: (data) => {
                        progress(data);
                    },
                    onEnd: () => {
                        this.chatDeferred.resolve();
                    },
                    onError: (error) => {
                        this.aiReporter.end(request.sessionId + '_' + request.requestId, {
                            message: error.message,
                            success: false,
                        });
                    },
                });
                await this.chatDeferred.promise;
                return {};
            },
            provideSlashCommands: async (token) => this.chatFeatureRegistry
                .getAllSlashCommand()
                .map((s) => (Object.assign(Object.assign({}, s), { name: s.name, description: s.description || '' }))),
            provideChatWelcomeMessage: async (token) => undefined,
        }));
        queueMicrotask(() => {
            this.chatAgentService.updateAgent(ChatProxyService_1.AGENT_ID, {});
        });
    }
};
// 避免和插件注册的 agent id 冲突
ChatProxyService.AGENT_ID = 'Default_Chat_Agent_' + (0, ide_core_common_1.uuid)(6);
tslib_1.__decorate([
    (0, di_1.Autowired)(common_1.IChatAgentService),
    tslib_1.__metadata("design:type", Object)
], ChatProxyService.prototype, "chatAgentService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.AIBackSerivcePath),
    tslib_1.__metadata("design:type", Object)
], ChatProxyService.prototype, "aiBackService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.ChatFeatureRegistryToken),
    tslib_1.__metadata("design:type", chat_feature_registry_1.ChatFeatureRegistry)
], ChatProxyService.prototype, "chatFeatureRegistry", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(command_service_1.MonacoCommandRegistry),
    tslib_1.__metadata("design:type", command_service_1.MonacoCommandRegistry)
], ChatProxyService.prototype, "monacoCommandRegistry", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.ChatServiceToken),
    tslib_1.__metadata("design:type", chat_api_service_1.ChatService)
], ChatProxyService.prototype, "aiChatService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.IAIReporter),
    tslib_1.__metadata("design:type", Object)
], ChatProxyService.prototype, "aiReporter", void 0);
ChatProxyService = ChatProxyService_1 = tslib_1.__decorate([
    (0, di_1.Injectable)()
], ChatProxyService);
exports.ChatProxyService = ChatProxyService;
//# sourceMappingURL=chat-proxy.service.js.map