"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChatService = void 0;
const tslib_1 = require("tslib");
const di_1 = require("@opensumi/di");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const ide_main_layout_1 = require("@opensumi/ide-main-layout");
const common_1 = require("../../common");
const msg_history_manager_1 = require("../model/msg-history-manager");
let ChatService = class ChatService extends ide_core_common_1.Disposable {
    constructor() {
        super();
        this._onChatMessageLaunch = new ide_core_common_1.Emitter();
        this.onChatMessageLaunch = this._onChatMessageLaunch.event;
        this._onChatReplyMessageLaunch = new ide_core_common_1.Emitter();
        this.onChatReplyMessageLaunch = this._onChatReplyMessageLaunch.event;
        this.addDispose(this.msgHistoryManager);
    }
    /**
     * 显示聊天视图
     */
    showChatView() {
        this.mainLayoutService.toggleSlot(common_1.AI_CHAT_VIEW_ID, true);
    }
    sendMessage(data) {
        this.showChatView();
        this._onChatMessageLaunch.fire(data);
    }
    clearHistoryMessages() {
        this.msgHistoryManager.clearMessages();
    }
    /**
     * 主动以 ai role 的身份回复消息
     */
    sendReplyMessage(data) {
        this._onChatReplyMessageLaunch.fire(data);
    }
    getHistoryMessages() {
        return this.msgHistoryManager.getMessages();
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(msg_history_manager_1.MsgHistoryManager),
    tslib_1.__metadata("design:type", msg_history_manager_1.MsgHistoryManager)
], ChatService.prototype, "msgHistoryManager", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_main_layout_1.IMainLayoutService),
    tslib_1.__metadata("design:type", Object)
], ChatService.prototype, "mainLayoutService", void 0);
ChatService = tslib_1.__decorate([
    (0, di_1.Injectable)(),
    tslib_1.__metadata("design:paramtypes", [])
], ChatService);
exports.ChatService = ChatService;
//# sourceMappingURL=chat.api.service.js.map