"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AIChatView = void 0;
const tslib_1 = require("tslib");
const mobx_react_lite_1 = require("mobx-react-lite");
const React = tslib_1.__importStar(require("react"));
const react_chat_elements_1 = require("react-chat-elements");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const components_1 = require("@opensumi/ide-core-browser/lib/components");
const ai_native_1 = require("@opensumi/ide-core-browser/lib/components/ai-native");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const ide_main_layout_1 = require("@opensumi/ide-main-layout");
require("react-chat-elements/dist/main.css");
const common_1 = require("../../common");
const ChatEditor_1 = require("../components/ChatEditor");
const ChatInput_1 = require("../components/ChatInput");
const ChatMarkdown_1 = require("../components/ChatMarkdown");
const ChatReply_1 = require("../components/ChatReply");
const SlashCustomRender_1 = require("../components/SlashCustomRender");
const utils_1 = require("../components/utils");
const WelcomeMsg_1 = require("../components/WelcomeMsg");
const msg_history_manager_1 = require("../model/msg-history-manager");
const chat_proxy_service_1 = require("./chat-proxy.service");
const chat_module_less_1 = tslib_1.__importDefault(require("./chat.module.less"));
const SCROLL_CLASSNAME = 'chat_scroll';
exports.AIChatView = (0, mobx_react_lite_1.observer)(() => {
    const aiChatService = (0, ide_core_browser_1.useInjectable)(common_1.IChatInternalService);
    const chatApiService = (0, ide_core_browser_1.useInjectable)(ide_core_common_1.ChatServiceToken);
    const aiReporter = (0, ide_core_browser_1.useInjectable)(ide_core_common_1.IAIReporter);
    const chatAgentService = (0, ide_core_browser_1.useInjectable)(common_1.IChatAgentService);
    const chatFeatureRegistry = (0, ide_core_browser_1.useInjectable)(ide_core_common_1.ChatFeatureRegistryToken);
    const chatRenderRegistry = (0, ide_core_browser_1.useInjectable)(ide_core_common_1.ChatRenderRegistryToken);
    const layoutService = (0, ide_core_browser_1.useInjectable)(ide_main_layout_1.IMainLayoutService);
    const msgHistoryManager = (0, ide_core_browser_1.useInjectable)(msg_history_manager_1.MsgHistoryManager);
    const containerRef = React.useRef(null);
    const chatInputRef = React.useRef(null);
    const [messageListData, dispatchMessage] = React.useReducer((state, action) => {
        switch (action.type) {
            case 'add':
                return [...state, ...(action.payload || [])];
            case 'clear':
                return [];
            case 'init':
                return Array.isArray(action.payload) ? action.payload : [];
            default:
                return state;
        }
    }, []);
    const [loading, setLoading] = React.useState(false);
    const [agentId, setAgentId] = React.useState('');
    const [defaultAgentId, setDefaultAgentId] = React.useState('');
    const [command, setCommand] = React.useState('');
    const [theme, setTheme] = React.useState(null);
    const aiAssistantName = React.useMemo(() => (0, ide_core_common_1.localize)('aiNative.chat.ai.assistant.name'), []);
    const shortcutCommands = React.useMemo(() => chatFeatureRegistry.getAllShortcutSlashCommand(), [chatFeatureRegistry]);
    const ChatInputWrapperRender = React.useMemo(() => {
        if (chatRenderRegistry.chatInputRender) {
            return chatRenderRegistry.chatInputRender;
        }
        return ChatInput_1.ChatInput;
    }, [chatRenderRegistry.chatInputRender]);
    const firstMsg = React.useMemo(() => (0, utils_1.createMessageByAI)({
        id: (0, ide_core_common_1.uuid)(6),
        relationId: '',
        text: React.createElement(WelcomeMsg_1.WelcomeMessage, null),
    }), []);
    const scrollToBottom = React.useCallback(() => {
        if (containerRef && containerRef.current) {
            containerRef.current.scrollTop = Number.MAX_SAFE_INTEGER;
            // 出现滚动条时出现分割线
            if (containerRef.current.scrollHeight > containerRef.current.clientHeight) {
                containerRef.current.classList.add(SCROLL_CLASSNAME);
            }
        }
    }, [containerRef]);
    const handleDispatchMessage = React.useCallback((dispatch) => {
        dispatchMessage(dispatch);
        requestAnimationFrame(() => {
            scrollToBottom();
        });
    }, [dispatchMessage, scrollToBottom]);
    React.useEffect(() => {
        handleDispatchMessage({ type: 'init', payload: [firstMsg] });
    }, []);
    React.useEffect(() => {
        scrollToBottom();
    }, [loading]);
    React.useEffect(() => {
        const disposer = new ide_core_common_1.Disposable();
        disposer.addDispose(chatApiService.onChatMessageLaunch(async (message) => {
            var _a;
            if (message.immediate !== false) {
                if (loading) {
                    return;
                }
                await handleSend(message);
            }
            else {
                if (message.agentId) {
                    setAgentId(message.agentId);
                }
                if (message.command) {
                    setCommand(message.command);
                }
                (_a = chatInputRef === null || chatInputRef === void 0 ? void 0 : chatInputRef.current) === null || _a === void 0 ? void 0 : _a.setInputValue(message.message);
            }
        }));
        disposer.addDispose(chatApiService.onChatReplyMessageLaunch((chunk) => {
            const relationId = aiReporter.start(ide_core_common_1.AISerivceType.CustomReplay, {
                msgType: ide_core_common_1.AISerivceType.CustomReplay,
                message: chunk,
            });
            let renderContent = React.createElement(ChatMarkdown_1.ChatMarkdown, { markdown: chunk, fillInIncompleteTokens: true });
            if (chatRenderRegistry.chatAIRoleRender) {
                const ChatAIRoleRender = chatRenderRegistry.chatAIRoleRender;
                renderContent = React.createElement(ChatAIRoleRender, { content: chunk });
            }
            msgHistoryManager.addAssistantMessage({
                content: chunk,
            });
            const aiMessage = (0, utils_1.createMessageByAI)({
                id: (0, ide_core_common_1.uuid)(6),
                relationId,
                text: renderContent,
                className: chat_module_less_1.default.chat_with_more_actions,
            });
            handleDispatchMessage({ type: 'add', payload: [aiMessage] });
        }));
        return () => disposer.dispose();
    }, [chatApiService, chatRenderRegistry.chatAIRoleRender, msgHistoryManager]);
    React.useEffect(() => {
        const disposer = new ide_core_common_1.Disposable();
        disposer.addDispose(chatAgentService.onDidSendMessage((chunk) => {
            const relationId = aiReporter.start(ide_core_common_1.AISerivceType.Agent, {
                msgType: ide_core_common_1.AISerivceType.Agent,
                message: '',
            });
            msgHistoryManager.addAssistantMessage({
                content: chunk.content,
            });
            const notifyMessage = (0, utils_1.createMessageByAI)({
                id: (0, ide_core_common_1.uuid)(6),
                relationId,
                text: React.createElement(ChatReply_1.ChatNotify, { requestId: aiChatService.latestRequestId, chunk: chunk }),
            }, chat_module_less_1.default.chat_notify);
            handleDispatchMessage({ type: 'add', payload: [notifyMessage] });
        }));
        disposer.addDispose(chatAgentService.onDidChangeAgents(async () => {
            const newDefaultAgentId = chatAgentService.getDefaultAgentId();
            setDefaultAgentId(newDefaultAgentId !== null && newDefaultAgentId !== void 0 ? newDefaultAgentId : '');
        }));
        return () => disposer.dispose();
    }, [chatAgentService, msgHistoryManager, aiChatService]);
    const handleSlashCustomRender = React.useCallback(async (value) => {
        const { userMessage, relationId, requestId, render, startTime } = value;
        msgHistoryManager.addAssistantMessage({
            type: 'component',
            content: '',
        });
        const aiMessage = (0, utils_1.createMessageByAI)({
            id: (0, ide_core_common_1.uuid)(6),
            relationId,
            className: chat_module_less_1.default.chat_with_more_actions,
            text: (React.createElement(SlashCustomRender_1.SlashCustomRender, { userMessage: userMessage, startTime: startTime, relationId: relationId, requestId: requestId, renderContent: render })),
        });
        handleDispatchMessage({ type: 'add', payload: [aiMessage] });
    }, [containerRef, msgHistoryManager]);
    const handleAgentReply = React.useCallback(async (value) => {
        const { message, agentId, command } = value;
        const request = aiChatService.createRequest(message, agentId, command);
        if (!request) {
            return;
        }
        setLoading(true);
        aiChatService.setLatestRequestId(request.requestId);
        const ChatUserRoleRender = chatRenderRegistry.chatUserRoleRender;
        const startTime = Date.now();
        const relationId = aiReporter.start(ide_core_common_1.AISerivceType.Agent, {
            msgType: ide_core_common_1.AISerivceType.Agent,
            message: value.message,
        });
        msgHistoryManager.addUserMessage({
            content: message,
            agentId: agentId,
            agentCommand: command,
        });
        const visibleAgentId = agentId === chat_proxy_service_1.ChatProxyService.AGENT_ID ? '' : agentId;
        const userMessage = (0, utils_1.createMessageByUser)({
            id: (0, ide_core_common_1.uuid)(6),
            relationId,
            text: ChatUserRoleRender ? (React.createElement(ChatUserRoleRender, { content: message, agentId: visibleAgentId, command: command })) : (React.createElement(ChatEditor_1.CodeBlockWrapperInput, { relationId: relationId, text: message, agentId: visibleAgentId, command: command })),
        }, chat_module_less_1.default.chat_message_code);
        handleDispatchMessage({ type: 'add', payload: [userMessage] });
        if (agentId === chat_proxy_service_1.ChatProxyService.AGENT_ID && command) {
            const commandHandler = chatFeatureRegistry.getSlashCommandHandler(command);
            if (commandHandler && commandHandler.providerRender) {
                setLoading(false);
                return handleSlashCustomRender({
                    userMessage: message,
                    render: commandHandler.providerRender,
                    relationId,
                    requestId: request.requestId,
                    startTime,
                });
            }
        }
        const aiMessage = (0, utils_1.createMessageByAI)({
            id: (0, ide_core_common_1.uuid)(6),
            relationId,
            className: chat_module_less_1.default.chat_with_more_actions,
            text: (React.createElement(ChatReply_1.ChatReply, { relationId: relationId, request: request, startTime: startTime, onDidChange: () => {
                    scrollToBottom();
                }, onDone: () => {
                    setLoading(false);
                }, onRegenerate: () => {
                    aiChatService.sendRequest(request, true);
                } })),
        });
        aiChatService.sendRequest(request);
        handleDispatchMessage({ type: 'add', payload: [aiMessage] });
    }, [chatRenderRegistry, chatRenderRegistry.chatUserRoleRender, msgHistoryManager, scrollToBottom]);
    const handleSend = React.useCallback(async (value) => {
        const { message, command } = value;
        const agentId = value.agentId ? value.agentId : chat_proxy_service_1.ChatProxyService.AGENT_ID;
        return handleAgentReply({ message, agentId, command });
    }, []);
    const handleClear = React.useCallback(() => {
        var _a;
        aiChatService.clearSessionModel();
        chatApiService.clearHistoryMessages();
        (_a = containerRef === null || containerRef === void 0 ? void 0 : containerRef.current) === null || _a === void 0 ? void 0 : _a.classList.remove(SCROLL_CLASSNAME);
        handleDispatchMessage({ type: 'init', payload: [firstMsg] });
    }, [messageListData]);
    const handleShortcutCommandClick = (commandModel) => {
        if (loading) {
            return;
        }
        setTheme(commandModel.nameWithSlash);
        setAgentId(commandModel.agentId);
        setCommand(commandModel.command);
    };
    const handleCloseChatView = React.useCallback(() => {
        layoutService.toggleSlot(common_1.AI_CHAT_VIEW_ID);
    }, [layoutService]);
    return (React.createElement("div", { id: chat_module_less_1.default.ai_chat_view },
        React.createElement("div", { className: chat_module_less_1.default.header_container },
            React.createElement("div", { className: chat_module_less_1.default.left },
                React.createElement("span", { className: chat_module_less_1.default.title }, aiAssistantName)),
            React.createElement("div", { className: chat_module_less_1.default.right },
                React.createElement(components_1.Popover, { overlayClassName: chat_module_less_1.default.popover_icon, id: 'ai-chat-header-clear', title: (0, ide_core_common_1.localize)('aiNative.operate.clear.title') },
                    React.createElement(ai_native_1.EnhanceIcon, { wrapperClassName: chat_module_less_1.default.action_btn, className: (0, ide_core_browser_1.getIcon)('clear'), onClick: handleClear })),
                React.createElement(components_1.Popover, { overlayClassName: chat_module_less_1.default.popover_icon, id: 'ai-chat-header-close', position: components_1.PopoverPosition.left, title: (0, ide_core_common_1.localize)('aiNative.operate.close.title') },
                    React.createElement(ai_native_1.EnhanceIcon, { wrapperClassName: chat_module_less_1.default.action_btn, className: (0, ide_core_browser_1.getIcon)('window-close'), onClick: handleCloseChatView })))),
        React.createElement("div", { className: chat_module_less_1.default.body_container },
            React.createElement("div", { className: chat_module_less_1.default.left_bar, id: 'ai_chat_left_container' },
                React.createElement("div", { className: chat_module_less_1.default.chat_container, ref: containerRef },
                    React.createElement(react_chat_elements_1.MessageList, { className: chat_module_less_1.default.message_list, lockable: true, toBottomHeight: '100%', 
                        // @ts-ignore
                        dataSource: messageListData })),
                React.createElement("div", { className: chat_module_less_1.default.chat_input_wrap },
                    React.createElement("div", { className: chat_module_less_1.default.header_operate },
                        React.createElement("div", { className: chat_module_less_1.default.header_operate_left }, shortcutCommands.map((command) => (React.createElement(components_1.Popover, { id: `ai-chat-shortcut-${command.name}`, key: `ai-chat-shortcut-${command.name}`, title: command.tooltip || command.name },
                            React.createElement("div", { className: chat_module_less_1.default.tag, onClick: () => handleShortcutCommandClick(command) }, command.name))))),
                        React.createElement("div", { className: chat_module_less_1.default.header_operate_right })),
                    React.createElement(ChatInputWrapperRender, { onSend: (value, agentId, command) => handleSend({ message: value, agentId, command }), disabled: loading, enableOptions: true, theme: theme, setTheme: setTheme, agentId: agentId, setAgentId: setAgentId, defaultAgentId: defaultAgentId, command: command, setCommand: setCommand, ref: chatInputRef }))))));
});
//# sourceMappingURL=chat.view.js.map