"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CodeBlockWrapperInput = exports.CodeBlockWrapper = exports.CodeEditorWithHighlight = void 0;
const tslib_1 = require("tslib");
const capitalize_1 = tslib_1.__importDefault(require("lodash/capitalize"));
const react_1 = tslib_1.__importStar(require("react"));
const react_highlight_1 = tslib_1.__importDefault(require("react-highlight"));
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const components_1 = require("@opensumi/ide-core-browser/lib/components");
const ai_native_1 = require("@opensumi/ide-core-browser/lib/components/ai-native");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const editor_collection_service_1 = require("@opensumi/ide-editor/lib/browser/editor-collection.service");
const command_service_1 = require("@opensumi/ide-editor/lib/browser/monaco-contrib/command/command.service");
const ide_theme_1 = require("@opensumi/ide-theme");
const components_module_less_1 = tslib_1.__importDefault(require("./components.module.less"));
const highLight_1 = require("./highLight");
require("./highlightTheme.less");
const CodeEditorWithHighlight = ({ input, language, relationId }) => {
    const ref = react_1.default.useRef(null);
    const monacoCommandRegistry = (0, ide_core_browser_1.useInjectable)(command_service_1.MonacoCommandRegistry);
    const clipboardService = (0, ide_core_browser_1.useInjectable)(ide_core_browser_1.IClipboardService);
    const workbenchThemeService = (0, ide_core_browser_1.useInjectable)(ide_theme_1.IThemeService);
    const aiReporter = (0, ide_core_browser_1.useInjectable)(ide_core_common_1.IAIReporter);
    const [isCoping, setIsCoping] = (0, react_1.useState)(false);
    const useUUID = (0, react_1.useMemo)(() => (0, ide_core_browser_1.uuid)(12), [ref, ref.current]);
    (0, react_1.useEffect)(() => {
        const doToggleTheme = (newTheme) => {
            if (newTheme.type === 'dark' || newTheme.type === 'hcDark') {
                import('highlight.js/styles/a11y-dark.css');
            }
            else if (newTheme.type === 'light' || newTheme.type === 'hcLight') {
                import('highlight.js/styles/a11y-light.css');
            }
        };
        const dispose = workbenchThemeService.onThemeChange((newTheme) => {
            doToggleTheme(newTheme);
        });
        const currentTheme = workbenchThemeService.getCurrentThemeSync();
        doToggleTheme(currentTheme);
        return () => dispose.dispose();
    }, []);
    const handleCopy = (0, react_1.useCallback)(async () => {
        setIsCoping(true);
        await clipboardService.writeText(input);
        aiReporter.end(relationId, { copy: true });
        (0, ide_core_common_1.runWhenIdle)(() => {
            setIsCoping(false);
        }, 1000);
    }, [clipboardService, input, relationId]);
    const handleInsert = (0, react_1.useCallback)(() => {
        const editor = monacoCommandRegistry.getActiveCodeEditor();
        if (editor) {
            const selection = editor.getSelection();
            if (selection) {
                (0, editor_collection_service_1.insertSnippetWithMonacoEditor)(editor, input, [selection], { undoStopBefore: false, undoStopAfter: false });
                aiReporter.end(relationId, { insert: true });
            }
        }
    }, [monacoCommandRegistry]);
    return (react_1.default.createElement("div", { className: components_module_less_1.default.monaco_wrapper },
        react_1.default.createElement("div", { className: components_module_less_1.default.action_toolbar },
            react_1.default.createElement(components_1.Popover, { id: `ai-chat-inser-${useUUID}`, title: (0, ide_core_common_1.localize)('aiNative.chat.code.insert') },
                react_1.default.createElement(ai_native_1.EnhanceIcon, { className: (0, ide_core_browser_1.getIcon)('insert'), onClick: () => handleInsert() })),
            react_1.default.createElement(components_1.Popover, { id: `ai-chat-copy-${useUUID}`, title: (0, ide_core_common_1.localize)(isCoping ? 'aiNative.chat.code.copy.success' : 'aiNative.chat.code.copy') },
                react_1.default.createElement(ai_native_1.EnhanceIcon, { className: (0, ide_core_browser_1.getIcon)('copy'), onClick: () => handleCopy() }))),
        react_1.default.createElement(react_highlight_1.default, { language: language, ref: ref, className: components_module_less_1.default.editor }, input)));
};
exports.CodeEditorWithHighlight = CodeEditorWithHighlight;
const CodeBlock = ({ content = '', relationId, renderText, }) => {
    const rgInlineCode = /`([^`]+)`/g;
    const rgBlockCode = /```([^]+?)```/g;
    const rgBlockCodeBefore = /```([^]+)?/g;
    const renderCodeEditor = (content) => {
        const language = content.split('\n')[0].trim().toLowerCase();
        const heighLightLang = highLight_1.highLightLanguageSupport.find((lang) => lang === language) || 'plaintext';
        content = content.replace(/.*?\n/, '');
        content = content.trim();
        return (react_1.default.createElement("div", { className: components_module_less_1.default.code_block },
            react_1.default.createElement("div", { className: components_module_less_1.default.code_language }, (0, capitalize_1.default)(heighLightLang)),
            react_1.default.createElement(exports.CodeEditorWithHighlight, { input: content, language: language, relationId: relationId })));
    };
    const render = (0, react_1.useMemo)(() => {
        const blocks = content.split(rgBlockCode);
        const renderedContent = [];
        blocks.map((block, index) => {
            if (index % 2 === 0) {
                block.split(rgInlineCode).map((text, index) => {
                    if (index % 2 === 0) {
                        if (text.includes('```')) {
                            const cutchunk = text.split(rgBlockCodeBefore).filter(Boolean);
                            if (cutchunk.length === 2) {
                                renderedContent.push(cutchunk[0]);
                                renderedContent.push(renderCodeEditor(cutchunk[1]));
                                return;
                            }
                        }
                        if (renderText) {
                            renderedContent.push(renderText(text));
                        }
                        else {
                            renderedContent.push(text);
                        }
                    }
                    else {
                        renderedContent.push(react_1.default.createElement("span", { className: components_module_less_1.default.code_inline, key: index }, text));
                    }
                });
            }
            else {
                renderedContent.push(renderCodeEditor(block));
            }
        });
        return renderedContent;
    }, [content, renderText]);
    return react_1.default.createElement(react_1.default.Fragment, null, render);
};
const CodeBlockWrapper = ({ text, renderText, relationId, }) => (react_1.default.createElement("div", { className: components_module_less_1.default.ai_chat_code_wrapper },
    react_1.default.createElement("div", { className: components_module_less_1.default.render_text },
        react_1.default.createElement(CodeBlock, { content: text, renderText: renderText, relationId: relationId }))));
exports.CodeBlockWrapper = CodeBlockWrapper;
const CodeBlockWrapperInput = ({ text, relationId, agentId, command, }) => {
    const chatFeatureRegistry = (0, ide_core_browser_1.useInjectable)(ide_core_common_1.ChatFeatureRegistryToken);
    const [tag, setTag] = (0, react_1.useState)('');
    const [txt, setTxt] = (0, react_1.useState)(text);
    react_1.default.useEffect(() => {
        const { value, nameWithSlash } = chatFeatureRegistry.parseSlashCommand(text);
        if (nameWithSlash) {
            setTag(nameWithSlash);
            setTxt(value);
            return;
        }
    }, [text, chatFeatureRegistry]);
    return (react_1.default.createElement("div", { className: components_module_less_1.default.ai_chat_code_wrapper },
        react_1.default.createElement("div", { className: components_module_less_1.default.render_text },
            tag && (react_1.default.createElement("div", { className: components_module_less_1.default.tag_wrapper },
                react_1.default.createElement("span", { className: components_module_less_1.default.tag }, tag))),
            agentId && (react_1.default.createElement("div", { className: components_module_less_1.default.tag, style: { marginRight: 4 } },
                "@",
                agentId)),
            command && react_1.default.createElement("div", { className: components_module_less_1.default.tag },
                "/ ",
                command),
            react_1.default.createElement(CodeBlock, { content: txt, relationId: relationId }))));
};
exports.CodeBlockWrapperInput = CodeBlockWrapperInput;
//# sourceMappingURL=ChatEditor.js.map