"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fillInIncompleteTokens = exports.postProcessCodeBlockLanguageId = exports.ChatMarkdown = void 0;
const tslib_1 = require("tslib");
const classnames_1 = tslib_1.__importDefault(require("classnames"));
const react_1 = tslib_1.__importStar(require("react"));
const client_1 = tslib_1.__importDefault(require("react-dom/client"));
const utils_1 = require("@opensumi/ide-components/lib/utils");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const id_generator_1 = require("@opensumi/ide-core-common/lib/id-generator");
const strings_1 = require("@opensumi/ide-utils/lib/strings");
const htmlContent_1 = require("@opensumi/monaco-editor-core/esm/vs/base/common/htmlContent");
const ChatEditor_1 = require("./ChatEditor");
const components_module_less_1 = tslib_1.__importDefault(require("./components.module.less"));
const ChatMarkdown = (props) => {
    const ref = (0, react_1.useRef)(null);
    const appConfig = (0, ide_core_browser_1.useInjectable)(ide_core_browser_1.AppConfig);
    (0, react_1.useEffect)(() => {
        var _a, _b;
        const element = ref.current;
        if (!element) {
            return;
        }
        const codeRenderedElements = new Map();
        const markdown = typeof props.markdown === 'string' ? new htmlContent_1.MarkdownString(props.markdown) : props.markdown;
        const renderer = new utils_1.marked.Renderer();
        renderer.link = (href, title, text) => `<a rel="noopener" target="_blank" href="${href}" target="${href}" title="${title || href}">${text}</a>`;
        renderer.code = (code, lang) => {
            const id = id_generator_1.defaultGenerator.nextId();
            const container = document.createElement('div');
            const language = postProcessCodeBlockLanguageId(lang);
            const dom = client_1.default.createRoot(container);
            dom.render(react_1.default.createElement(ide_core_browser_1.ConfigProvider, { value: appConfig },
                react_1.default.createElement("div", { className: components_module_less_1.default.code_block },
                    react_1.default.createElement("div", { className: components_module_less_1.default.code_language }, language),
                    react_1.default.createElement(ChatEditor_1.CodeEditorWithHighlight, { input: code, language: language, relationId: props.relationId }))));
            codeRenderedElements.set(id, { container, root: dom });
            return `<div class="code" data-code="${id}">${(0, strings_1.escape)(code)}</div>`;
        };
        renderer.codespan = (code) => `<code class=${components_module_less_1.default.code_inline}>${code}</code>`;
        const markedOptions = (_a = props.markedOptions) !== null && _a !== void 0 ? _a : {};
        markedOptions.renderer = renderer;
        let value = (_b = markdown.value) !== null && _b !== void 0 ? _b : '';
        if (value.length > 100000) {
            value = `${value.slice(0, 100000)}…`;
        }
        let renderedMarkdown;
        if (props.fillInIncompleteTokens) {
            const opts = Object.assign(Object.assign({}, utils_1.marked.defaults), markedOptions);
            const tokens = utils_1.marked.lexer(value, opts);
            const newTokens = fillInIncompleteTokens(tokens);
            renderedMarkdown = utils_1.marked.parser(newTokens, opts);
        }
        else {
            renderedMarkdown = utils_1.marked.parse(value, markedOptions);
        }
        element.innerHTML = renderedMarkdown;
        const codePlaceholderElements = element.querySelectorAll('div[data-code]');
        codePlaceholderElements.forEach((placeholderElement) => {
            var _a;
            const renderedElement = codeRenderedElements.get((_a = placeholderElement.dataset['code']) !== null && _a !== void 0 ? _a : '');
            if (renderedElement && renderedElement.container) {
                placeholderElement.innerText = '';
                placeholderElement.append(renderedElement.container);
            }
        });
        return () => {
            if (codeRenderedElements.size > 0) {
                codeRenderedElements.forEach(({ root }) => {
                    requestAnimationFrame(() => {
                        root.unmount();
                    });
                });
            }
        };
    }, [props.markdown]);
    return react_1.default.createElement("div", { className: (0, classnames_1.default)(components_module_less_1.default.markdown_container, props.className), ref: ref });
};
exports.ChatMarkdown = ChatMarkdown;
function postProcessCodeBlockLanguageId(lang) {
    if (!lang) {
        return '';
    }
    const parts = lang.split(/[\s+|:|,|\{|\?]/, 1);
    if (parts.length) {
        return parts[0];
    }
    return lang;
}
exports.postProcessCodeBlockLanguageId = postProcessCodeBlockLanguageId;
function fillInIncompleteTokens(tokens) {
    let i;
    let newTokens;
    for (i = 0; i < tokens.length; i++) {
        const token = tokens[i];
        // 代码块补全，完整的代码块 type=code
        if (token.type === 'paragraph' && token.raw.match(/(\n|^)```/)) {
            newTokens = completeCodeBlock(tokens.slice(i));
            break;
        }
        // 表格补全，完整的表格 type=table
        if (token.type === 'paragraph' && token.raw.match(/(\n|^)\|/)) {
            newTokens = completeTable(tokens.slice(i));
            break;
        }
        // 单行 token 补全，如 `a **a
        if (token.type === 'paragraph' && i === tokens.length - 1) {
            // Only operates on a single token, because any newline that follows this should break these patterns
            const newToken = completeSingleLinePattern(token);
            if (newToken) {
                newTokens = [newToken];
                break;
            }
        }
    }
    if (newTokens) {
        const newTokensList = [...tokens.slice(0, i), ...newTokens];
        newTokensList.links = tokens.links;
        return newTokensList;
    }
    return tokens;
}
exports.fillInIncompleteTokens = fillInIncompleteTokens;
function completeCodeBlock(tokens) {
    const mergedRawText = mergeRawTokenText(tokens);
    return utils_1.marked.lexer(mergedRawText + '\n```');
}
function completeCodespan(token) {
    return completeWithString(token, '`');
}
function completeStar(tokens) {
    return completeWithString(tokens, '*');
}
function completeUnderscore(tokens) {
    return completeWithString(tokens, '_');
}
function completeLinkTarget(tokens) {
    return completeWithString(tokens, ')');
}
function completeLinkText(tokens) {
    return completeWithString(tokens, '](about:blank)');
}
function completeDoublestar(tokens) {
    return completeWithString(tokens, '**');
}
function completeDoubleUnderscore(tokens) {
    return completeWithString(tokens, '__');
}
function completeWithString(tokens, closingString) {
    const mergedRawText = mergeRawTokenText(Array.isArray(tokens) ? tokens : [tokens]);
    // If it was completed correctly, this should be a single token.
    // Expecting either a Paragraph or a List
    return utils_1.marked.lexer(mergedRawText + closingString)[0];
}
function completeTable(tokens) {
    const mergedRawText = mergeRawTokenText(tokens);
    const lines = mergedRawText.split('\n');
    let numCols = 0;
    let hasSeparatorRow = false;
    for (let i = 0; i < lines.length; i++) {
        const line = lines[i].trim();
        // 确定第一行有多少个 |
        if (typeof numCols === 'undefined' && line.match(/^\s*\|/)) {
            const line1Matches = line.match(/(\|[^\|]+)(?=\||$)/g);
            if (line1Matches) {
                numCols = line1Matches.length;
            }
        }
        else if (typeof numCols === 'number') {
            // 确定最后一行为分割行，否则表格可能不完整，此时不做任何渲染
            if (line.match(/^\s*\|/) && i === lines.length - 1) {
                hasSeparatorRow = true;
            }
            else {
                return undefined;
            }
        }
    }
    if (numCols > 0) {
        const prefixText = hasSeparatorRow ? lines.slice(0, -1).join('\n') : mergedRawText;
        const line1EndsInPipe = !!prefixText.match(/\|\s*$/);
        const newRawText = prefixText + (line1EndsInPipe ? '' : '|') + `\n|${' --- |'.repeat(numCols)}`;
        return utils_1.marked.lexer(newRawText);
    }
    return undefined;
}
function mergeRawTokenText(tokens) {
    return tokens.reduce((mergedTokenText, token) => mergedTokenText + token.raw, '');
}
function completeSingleLinePattern(token) {
    for (const { type, raw } of token.tokens) {
        if (type !== 'text') {
            continue;
        }
        const lines = raw.split('\n');
        const lastLine = lines[lines.length - 1];
        const patterns = [
            { condition: lastLine.includes('`'), action: completeCodespan },
            { condition: lastLine.includes('**'), action: completeDoublestar },
            { condition: lastLine.match(/\*\w/), action: completeStar },
            { condition: lastLine.match(/(^|\s)__\w/), action: completeDoubleUnderscore },
            { condition: lastLine.match(/(^|\s)_\w/), action: completeUnderscore },
            { condition: lastLine.match(/(^|\s)\[.*\]\(\w*/), action: completeLinkTarget },
            { condition: lastLine.match(/(^|\s)\[\w/), action: completeLinkText },
        ];
        for (const pattern of patterns) {
            if (pattern.condition) {
                return pattern.action(token);
            }
        }
    }
    return undefined;
}
//# sourceMappingURL=ChatMarkdown.js.map