"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WelcomeMessage = void 0;
const tslib_1 = require("tslib");
const React = tslib_1.__importStar(require("react"));
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const components_1 = require("@opensumi/ide-core-browser/lib/components");
const event_1 = require("@opensumi/ide-core-browser/lib/dom/event");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const htmlContent_1 = require("@opensumi/monaco-editor-core/esm/vs/base/common/htmlContent");
require("react-chat-elements/dist/main.css");
const common_1 = require("../../common");
const ChatMarkdown_1 = require("../components/ChatMarkdown");
const ChatThinking_1 = require("../components/ChatThinking");
const utils_1 = require("../components/utils");
const components_module_less_1 = tslib_1.__importDefault(require("./components.module.less"));
const WelcomeMessage = () => {
    var _a, _b;
    const aiChatService = (0, ide_core_browser_1.useInjectable)(ide_core_common_1.ChatServiceToken);
    const chatAgentService = (0, ide_core_browser_1.useInjectable)(common_1.IChatAgentService);
    const chatFeatureRegistry = (0, ide_core_browser_1.useInjectable)(ide_core_common_1.ChatFeatureRegistryToken);
    const chatRenderRegistry = (0, ide_core_browser_1.useInjectable)(ide_core_common_1.ChatRenderRegistryToken);
    const [sampleQuestions, setSampleQuestions] = React.useState([]);
    const welcomeSampleQuestions = React.useMemo(() => {
        if (!chatFeatureRegistry.chatWelcomeMessageModel) {
            return [];
        }
        const { sampleQuestions = [] } = chatFeatureRegistry.chatWelcomeMessageModel;
        return sampleQuestions.map(utils_1.extractIcon);
    }, [(_a = chatFeatureRegistry.chatWelcomeMessageModel) === null || _a === void 0 ? void 0 : _a.sampleQuestions]);
    const welcomeMessage = React.useMemo(() => {
        if (!chatFeatureRegistry.chatWelcomeMessageModel) {
            return '';
        }
        const { content } = chatFeatureRegistry.chatWelcomeMessageModel;
        return content;
    }, [(_b = chatFeatureRegistry.chatWelcomeMessageModel) === null || _b === void 0 ? void 0 : _b.content]);
    React.useEffect(() => {
        const disposer = chatAgentService.onDidChangeAgents(async () => {
            const sampleQuestions = await chatAgentService.getAllSampleQuestions();
            const lists = sampleQuestions.map(utils_1.extractIcon);
            setSampleQuestions(lists);
        });
        return () => disposer.dispose();
    }, []);
    if (!welcomeMessage) {
        return React.createElement(ChatThinking_1.ChatThinking, { showStop: false, thinkingText: (0, ide_core_common_1.localize)('aiNative.chat.welcome.loading.text') });
    }
    const allSampleQuestions = React.useMemo(() => welcomeSampleQuestions.concat(sampleQuestions), [welcomeSampleQuestions, sampleQuestions]);
    const welcomeRender = React.useMemo(() => {
        if (chatRenderRegistry.chatWelcomeRender) {
            const Render = chatRenderRegistry.chatWelcomeRender;
            return React.createElement(Render, { message: welcomeMessage, sampleQuestions: allSampleQuestions });
        }
        return (React.createElement("div", { className: components_module_less_1.default.chat_welcome_head },
            React.createElement("div", { className: components_module_less_1.default.chat_container_des }, (0, htmlContent_1.isMarkdownString)(welcomeMessage) ? React.createElement(ChatMarkdown_1.ChatMarkdown, { markdown: welcomeMessage }) : welcomeMessage),
            React.createElement("div", { className: components_module_less_1.default.chat_container_content }, allSampleQuestions.map((data, index) => {
                const node = (React.createElement("a", { className: components_module_less_1.default.link_item, onClick: (0, event_1.withPrevented)(() => {
                        aiChatService.sendMessage(chatAgentService.parseMessage(data.message));
                    }) },
                    data.icon ? React.createElement(components_1.Icon, { className: data.icon, style: { color: 'inherit', marginRight: '4px' } }) : '',
                    React.createElement("span", null, data.title)));
                return data.tooltip ? (React.createElement(components_1.Tooltip, { title: data.tooltip, key: index }, node)) : (React.createElement(React.Fragment, { key: index }, node));
            }))));
    }, [chatRenderRegistry.chatWelcomeRender, welcomeMessage, allSampleQuestions]);
    return welcomeRender;
};
exports.WelcomeMessage = WelcomeMessage;
//# sourceMappingURL=WelcomeMsg.js.map