import Parser from 'web-tree-sitter';
import * as monaco from '@opensumi/ide-monaco/lib/common';
import { IDisposable } from '@opensumi/ide-utils';
import { ICodeBlockInfo, IOtherBlockInfo } from './tree-sitter/language-facts/base';
export declare class LanguageParser implements IDisposable {
    private language;
    private parser;
    private parserLoaded;
    private lruCache;
    private wasmModuleManager;
    private languageFacts;
    private constructor();
    ready(): Promise<void>;
    getParser(): Parser;
    private initializeParser;
    /**
     * 从给定的位置开始，向上遍历 AST，找到最近的代码块
     * @param node 节点
     * @param position 位置
     * @returns 代码块
     */
    private findContainingCodeBlockWithPosition;
    private findFunctionCodeBlock;
    parseAST(model: monaco.ITextModel): Promise<Parser.SyntaxNode | undefined>;
    getSyntaxNodeAsPosition(model: monaco.ITextModel, cursor: number): Promise<Parser.SyntaxNode | null>;
    /**
     * 从给定的位置开始，找到最近的没有语法错误的代码块
     */
    findCodeBlockWithSyntaxError(sourceCode: string, range: monaco.IRange): Promise<IOtherBlockInfo | null>;
    provideCodeBlockInfo(model: monaco.ITextModel, position: monaco.Position): Promise<ICodeBlockInfo | null>;
    provideCodeBlockInfoInRange(model: monaco.ITextModel, range: monaco.IRange): Promise<ICodeBlockInfo | null>;
    dispose(): void;
}
//# sourceMappingURL=parser.d.ts.map