"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DiffResolveResultWidget = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const di_1 = require("@opensumi/di");
const containerPanel_1 = require("@opensumi/ide-core-browser/lib/components/ai-native/content-widget/containerPanel");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const monaco = tslib_1.__importStar(require("@opensumi/ide-monaco"));
const line_range_1 = require("@opensumi/ide-monaco/lib/browser/contrib/merge-editor/model/line-range");
const types_1 = require("@opensumi/ide-monaco/lib/browser/contrib/merge-editor/types");
const resultCodeEditor_1 = require("@opensumi/ide-monaco/lib/browser/contrib/merge-editor/view/editors/resultCodeEditor");
const resolve_result_widget_1 = require("@opensumi/ide-monaco/lib/browser/contrib/merge-editor/widget/resolve-result-widget");
const inline_diff_widget_1 = require("../widget/inline-diff/inline-diff-widget");
let DiffResolveResultWidget = class DiffResolveResultWidget extends resolve_result_widget_1.ResolveResultWidget {
    constructor(uid, codeEditor, lineRange, range, text) {
        super(uid, codeEditor, lineRange);
        this.uid = uid;
        this.codeEditor = codeEditor;
        this.lineRange = lineRange;
        this.range = range;
        this.text = text;
    }
    isRenderThumbs() {
        return true;
    }
    iconItems() {
        return [
            {
                icon: 'check',
                text: (0, ide_core_common_1.localize)('aiNative.inline.chat.operate.check.title'),
                onClick: () => {
                    var _a;
                    const modifiedValue = (_a = this.inlineDiffWidget) === null || _a === void 0 ? void 0 : _a.getModifiedValue();
                    this.codeEditor.launchConflictActionsEvent({
                        range: this.lineRange,
                        action: types_1.ACCEPT_CURRENT_ACTIONS,
                        reason: types_1.ECompleteReason.UserManual,
                        value: modifiedValue,
                    });
                },
            },
            {
                icon: 'discard',
                text: (0, ide_core_common_1.localize)('aiNative.operate.discard.title'),
                onClick: () => {
                    this.codeEditor.launchConflictActionsEvent({
                        range: this.lineRange,
                        action: types_1.REVOKE_ACTIONS,
                        reason: types_1.ECompleteReason.UserManual,
                    });
                },
            },
        ];
    }
    hide() {
        var _a;
        super.hide();
        this.codeEditor.editor.setHiddenAreas([], this.uid);
        (_a = this.inlineDiffWidget) === null || _a === void 0 ? void 0 : _a.dispose();
        this.inlineDiffWidget = undefined;
    }
    renderView() {
        const iconResultItems = this.iconItems();
        const isRenderThumbs = this.isRenderThumbs();
        const handleCloseClick = () => {
            this.codeEditor.launchConflictActionsEvent({
                range: this.lineRange,
                action: types_1.IGNORE_ACTIONS,
                reason: types_1.ECompleteReason.UserManual,
            });
        };
        // 渲染在下一行空行上，但是通过 css 下移 1/2 行
        const halfLineHeight = this.getLineHeight() / 2;
        const resultWidget = (react_1.default.createElement(containerPanel_1.ContentWidgetContainerPanel, { style: { transform: `translateY(${halfLineHeight}px)`, left: 2, display: 'flex', position: 'absolute' } },
            react_1.default.createElement(resolve_result_widget_1.WapperAIInlineResult, { id: this.uid, iconItems: iconResultItems, isRenderThumbs: isRenderThumbs, codeEditor: this.codeEditor, range: this.lineRange, closeClick: handleCloseClick, isRenderClose: true, disablePopover: true })));
        if (this.inlineDiffWidget) {
            this.inlineDiffWidget.dispose();
        }
        const { range } = this;
        // hidden full conflict area, but only show diff for current and the ai response
        this.inlineDiffWidget = this.injector.get(inline_diff_widget_1.InlineDiffWidget, [
            this.uid,
            this.codeEditor.editor,
            range,
            this.text,
            this.lineRange.toInclusiveRange(),
        ]);
        this.inlineDiffWidget.setResolveResultWidget(resultWidget);
        this.inlineDiffWidget.create();
        this.inlineDiffWidget.show(range, range.endLineNumber - range.startLineNumber + 1);
        return null;
    }
    id() {
        return `${types_1.AIResolveConflictContentWidget}_${this.uid}`;
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(di_1.INJECTOR_TOKEN),
    tslib_1.__metadata("design:type", di_1.Injector)
], DiffResolveResultWidget.prototype, "injector", void 0);
DiffResolveResultWidget = tslib_1.__decorate([
    (0, di_1.Injectable)({ multiple: true }),
    tslib_1.__metadata("design:paramtypes", [String, resultCodeEditor_1.ResultCodeEditor,
        line_range_1.LineRange, Object, String])
], DiffResolveResultWidget);
exports.DiffResolveResultWidget = DiffResolveResultWidget;
//# sourceMappingURL=override-resolve-result-widget.js.map