"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AIInlineContentWidget = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const di_1 = require("@opensumi/di");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const monaco = tslib_1.__importStar(require("@opensumi/ide-monaco"));
const browser_1 = require("@opensumi/ide-monaco/lib/browser");
const BaseInlineContentWidget_1 = require("@opensumi/ide-monaco/lib/browser/ai-native/BaseInlineContentWidget");
const ai_native_contextkey_service_1 = require("../../contextkey/ai-native.contextkey.service");
const inline_chat_controller_1 = require("./inline-chat-controller");
const inline_chat_service_1 = require("./inline-chat.service");
let AIInlineContentWidget = class AIInlineContentWidget extends BaseInlineContentWidget_1.ReactInlineContentWidget {
    constructor(editor) {
        super(editor);
        this.editor = editor;
        this.originTop = 0;
        this._onActionClickEmitter = new ide_core_common_1.Emitter();
        this.onActionClick = this._onActionClickEmitter.event;
        this.aiNativeContextKey = this.injector.get(ai_native_contextkey_service_1.AINativeContextKey, [this.editor._contextKeyService]);
        this.addDispose(this.editor.onDidLayoutChange(() => {
            if (this.isOutOfArea()) {
                this.dispose();
            }
        }));
    }
    dispose() {
        this.aiInlineChatService.launchChatStatus(0 /* EInlineChatStatus.READY */);
        super.dispose();
    }
    clickActionId(actionId, source) {
        this._onActionClickEmitter.fire({ actionId, source });
    }
    renderView() {
        return (react_1.default.createElement(inline_chat_controller_1.AIInlineChatController, { onClickActions: (id) => this.clickActionId(id, 'widget'), onClose: () => this.dispose() }));
    }
    async show(options) {
        super.show(options);
        this.aiNativeContextKey.inlineChatIsVisible.set(true);
    }
    getDomNode() {
        const domNode = super.getDomNode();
        requestAnimationFrame(() => {
            domNode.style.padding = '6px';
            domNode.style.zIndex = ide_core_browser_1.StackingLevelStr.OverlayTop;
        });
        return domNode;
    }
    async hide() {
        this.aiNativeContextKey.inlineChatIsVisible.set(false);
        super.hide();
    }
    setOptions(options) {
        this.options = options;
    }
    offsetTop(top) {
        if (this.originTop === 0) {
            const top = this.domNode.style.top;
            this.originTop = top ? parseInt(top, 10) : 0;
        }
        this.domNode.style.top = `${this.originTop + top}px`;
    }
    id() {
        return ide_core_common_1.AIInlineChatContentWidgetId;
    }
    getPosition() {
        const position = super.getPosition();
        if (position) {
            return position;
        }
        if (!this.options) {
            return null;
        }
        const { selection } = this.options;
        if (!selection) {
            return null;
        }
        const target = this.computePosition(selection);
        if (target) {
            return target;
        }
        return null;
    }
    /**
     * 获取指定行的最后一个非空白字符的列数
     */
    safeGetLineLastNonWhitespaceColumn(line) {
        const model = this.editor.getModel();
        if (line < 1) {
            line = 1;
        }
        const lineCount = model.getLineCount();
        if (line > lineCount) {
            line = lineCount;
        }
        return model.getLineLastNonWhitespaceColumn(line);
    }
    toAbovePosition(lineNumber, column) {
        return {
            position: new monaco.Position(lineNumber, column),
            preference: [
                browser_1.monacoBrowser.editor.ContentWidgetPositionPreference.ABOVE,
                browser_1.monacoBrowser.editor.ContentWidgetPositionPreference.BELOW,
            ],
        };
    }
    toBelowPosition(lineNumber, column) {
        return {
            position: new monaco.Position(lineNumber, column),
            preference: [
                browser_1.monacoBrowser.editor.ContentWidgetPositionPreference.BELOW,
                browser_1.monacoBrowser.editor.ContentWidgetPositionPreference.ABOVE,
            ],
        };
    }
    recheckPosition(lineNumber, column) {
        const preNonWhitespaceColumn = this.safeGetLineLastNonWhitespaceColumn(lineNumber - 1);
        const curNonWhitespaceColumn = this.safeGetLineLastNonWhitespaceColumn(lineNumber);
        const nextNonWhitespaceColumn = this.safeGetLineLastNonWhitespaceColumn(lineNumber + 1);
        let newPreference = [browser_1.monacoBrowser.editor.ContentWidgetPositionPreference.ABOVE];
        let newLineNumber = lineNumber;
        let newColumn = column;
        if (curNonWhitespaceColumn >= nextNonWhitespaceColumn) {
            newPreference = [browser_1.monacoBrowser.editor.ContentWidgetPositionPreference.BELOW];
        }
        else if (curNonWhitespaceColumn >= preNonWhitespaceColumn) {
            newPreference = [browser_1.monacoBrowser.editor.ContentWidgetPositionPreference.ABOVE];
        }
        else {
            newColumn = Math.min(preNonWhitespaceColumn, nextNonWhitespaceColumn);
            if (preNonWhitespaceColumn >= nextNonWhitespaceColumn) {
                newPreference = [browser_1.monacoBrowser.editor.ContentWidgetPositionPreference.BELOW];
                newLineNumber = lineNumber - 1;
            }
            else {
                newPreference = [browser_1.monacoBrowser.editor.ContentWidgetPositionPreference.ABOVE];
                newLineNumber = lineNumber + 1;
            }
        }
        if (lineNumber === 1 || lineNumber === 2) {
            newPreference = [browser_1.monacoBrowser.editor.ContentWidgetPositionPreference.BELOW];
        }
        return {
            position: new monaco.Position(newLineNumber, newColumn),
            preference: newPreference,
        };
    }
    isProtrudeAbove(line) {
        const currentLastNonWhitespace = this.safeGetLineLastNonWhitespaceColumn(line);
        return (currentLastNonWhitespace >= this.safeGetLineLastNonWhitespaceColumn(line - 1) &&
            currentLastNonWhitespace >= this.safeGetLineLastNonWhitespaceColumn(line - 2));
    }
    isProtrudeBelow(line) {
        const currentLastNonWhitespace = this.safeGetLineLastNonWhitespaceColumn(line);
        return (currentLastNonWhitespace >= this.safeGetLineLastNonWhitespaceColumn(line + 1) &&
            currentLastNonWhitespace >= this.safeGetLineLastNonWhitespaceColumn(line + 2));
    }
    /**
     * 动态计算要显示的位置
     * 1. 以选区里的光标作为顶点
     * 2. 靠近光标处周围没有字符的空白区域作为要显示的区域
     * 3. 显示的区域方向在右侧，左侧不考虑
     */
    computePosition(selection) {
        const startPosition = selection.getStartPosition();
        const endPosition = selection.getEndPosition();
        const model = this.editor.getModel();
        if (!model) {
            return null;
        }
        const cursorPosition = selection.getPosition();
        const getCursorLastNonWhitespaceColumn = this.safeGetLineLastNonWhitespaceColumn(cursorPosition.lineNumber);
        let targetLine = null;
        let direction = null;
        // 用户只选中了一行
        if (startPosition.lineNumber === endPosition.lineNumber) {
            return this.recheckPosition(cursorPosition.lineNumber, cursorPosition.column);
        }
        // 用户选中了多行，光标在选中的开始位置
        if (cursorPosition.equals(startPosition)) {
            const getMaxLastWhitespaceColumn = Math.max(this.safeGetLineLastNonWhitespaceColumn(cursorPosition.lineNumber - 1), this.safeGetLineLastNonWhitespaceColumn(cursorPosition.lineNumber - 2));
            // 如果上面两行的最后一个非空白字符的列数小于当前行的最后一个非空白字符的列数 + 10
            // 则直接显示在上面
            if (getMaxLastWhitespaceColumn < getCursorLastNonWhitespaceColumn + 10) {
                return this.toAbovePosition(cursorPosition.lineNumber, getMaxLastWhitespaceColumn + 1);
            }
            for (let i = startPosition.lineNumber; i <= endPosition.lineNumber; i++) {
                if (this.isProtrudeAbove(i)) {
                    targetLine = i;
                    direction = 'above';
                    break;
                }
                if (this.isProtrudeBelow(i)) {
                    targetLine = i;
                    direction = 'below';
                    break;
                }
            }
        }
        else if (cursorPosition.equals(endPosition)) {
            const getMaxLastWhitespaceColumn = Math.max(this.safeGetLineLastNonWhitespaceColumn(cursorPosition.lineNumber + 1), this.safeGetLineLastNonWhitespaceColumn(cursorPosition.lineNumber + 2));
            if (getMaxLastWhitespaceColumn < getCursorLastNonWhitespaceColumn + 10) {
                return this.toBelowPosition(cursorPosition.lineNumber, getMaxLastWhitespaceColumn + 1);
            }
            for (let i = endPosition.lineNumber; i >= startPosition.lineNumber; i--) {
                if (this.isProtrudeBelow(i)) {
                    targetLine = i;
                    direction = 'below';
                    break;
                }
                if (this.isProtrudeAbove(i)) {
                    targetLine = i;
                    direction = 'above';
                    break;
                }
            }
        }
        if (targetLine && direction) {
            const column = this.safeGetLineLastNonWhitespaceColumn(targetLine) + 1;
            if (direction === 'below') {
                return this.toBelowPosition(targetLine, column);
            }
            return this.toAbovePosition(targetLine, column);
        }
        return this.recheckPosition(cursorPosition.lineNumber, this.safeGetLineLastNonWhitespaceColumn(cursorPosition.lineNumber));
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(di_1.INJECTOR_TOKEN),
    tslib_1.__metadata("design:type", di_1.Injector)
], AIInlineContentWidget.prototype, "injector", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.IAIInlineChatService),
    tslib_1.__metadata("design:type", inline_chat_service_1.AIInlineChatService)
], AIInlineContentWidget.prototype, "aiInlineChatService", void 0);
AIInlineContentWidget = tslib_1.__decorate([
    (0, di_1.Injectable)({ multiple: true }),
    tslib_1.__metadata("design:paramtypes", [Object])
], AIInlineContentWidget);
exports.AIInlineContentWidget = AIInlineContentWidget;
//# sourceMappingURL=inline-content-widget.js.map