"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InlineDiffWidget = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const react_dom_1 = require("react-dom");
const client_1 = tslib_1.__importDefault(require("react-dom/client"));
const di_1 = require("@opensumi/di");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const monaco = tslib_1.__importStar(require("@opensumi/ide-monaco"));
const monaco_api_1 = require("@opensumi/ide-monaco/lib/browser/monaco-api");
const model_1 = require("@opensumi/monaco-editor-core/esm/vs/editor/common/services/model");
const zoneWidget_1 = require("@opensumi/monaco-editor-core/esm/vs/editor/contrib/zoneWidget/browser/zoneWidget");
const standaloneServices_1 = require("@opensumi/monaco-editor-core/esm/vs/editor/standalone/browser/standaloneServices");
const inline_diff_widget_module_less_1 = tslib_1.__importDefault(require("./inline-diff-widget.module.less"));
const diffEditorOptions = {
    fixedOverflowWidgets: true,
    readOnly: false,
    enableSplitViewResizing: true,
    ignoreTrimWhitespace: true,
    renderSideBySide: true,
    lineNumbers: 'on',
    glyphMargin: true,
    scrollbar: { useShadows: false, alwaysConsumeMouseWheel: false, vertical: 'hidden' },
    scrollBeyondLastLine: false,
    renderMarginRevertIcon: true,
    renderOverviewRuler: false,
    rulers: undefined,
    overviewRulerBorder: undefined,
    overviewRulerLanes: 0,
    padding: { top: 0, bottom: 0 },
    folding: false,
    diffCodeLens: false,
    stickyScroll: { enabled: false },
    minimap: { enabled: false },
    automaticLayout: true,
};
const DiffContentProvider = react_1.default.memo((props) => {
    const { dto, onMaxLineCount, editor, onReady } = props;
    const monacoService = (0, ide_core_browser_1.useInjectable)(ide_core_browser_1.MonacoService);
    const editorRef = (0, react_1.useRef)(null);
    (0, react_1.useEffect)(() => {
        if (!dto) {
            return;
        }
        const model = editor.getModel();
        if (!model) {
            return;
        }
        const { range, modifiedValue } = dto;
        const codeValueInRange = model.getValueInRange(range);
        const diffEditor = monacoService.createDiffEditor(editorRef.current, Object.assign(Object.assign({}, diffEditorOptions), { lineDecorationsWidth: editor.getLayoutInfo().decorationsWidth, lineNumbersMinChars: editor.getOption(monaco.editor.EditorOption.lineNumbersMinChars) }));
        const modelService = standaloneServices_1.StandaloneServices.get(model_1.IModelService);
        const languageSelection = { languageId: model.getLanguageId(), onDidChange: ide_core_browser_1.Event.None };
        const originalModel = modelService.createModel(codeValueInRange, languageSelection);
        const modifiedModel = modelService.createModel(modifiedValue, languageSelection);
        diffEditor.setModel({
            original: originalModel,
            modified: modifiedModel,
        });
        diffEditor.revealLine(range.startLineNumber, monaco.editor.ScrollType.Immediate);
        if (onReady) {
            onReady({
                getModifiedValue: () => modifiedModel.getValue(),
            });
        }
        if (onMaxLineCount) {
            const originalEditor = diffEditor.getOriginalEditor();
            const modifiedEditor = diffEditor.getModifiedEditor();
            const originContentHeight = originalEditor.getContentHeight();
            const originLineCount = originContentHeight / originalEditor.getOption(monaco_api_1.monaco.editor.EditorOption.lineHeight);
            const modifiedContentHeight = modifiedEditor.getContentHeight();
            const modifiedLineCount = modifiedContentHeight / modifiedEditor.getOption(monaco_api_1.monaco.editor.EditorOption.lineHeight);
            onMaxLineCount(Math.max(originLineCount, modifiedLineCount) + 1);
        }
        return () => {
            if (diffEditor) {
                diffEditor.dispose();
            }
        };
    }, [dto]);
    return react_1.default.createElement("div", { ref: editorRef, className: inline_diff_widget_module_less_1.default.diff_editor_widget });
});
let InlineDiffWidget = class InlineDiffWidget extends zoneWidget_1.ZoneWidget {
    _fillContainer(container) {
        this.setCssClass('inline-diff-widget');
        this.root = client_1.default.createRoot(container);
        let portal = null;
        if (this._resolveResultWidget) {
            if (container.parentNode) {
                const resultContainer = document.createElement('div');
                requestAnimationFrame(() => {
                    resultContainer.className = inline_diff_widget_module_less_1.default.ai_diff_editor_resolve_result_widget;
                    const layoutInfo = this.editor.getLayoutInfo();
                    resultContainer.style.width = `${layoutInfo.contentWidth}px`;
                    resultContainer.style.left = `${layoutInfo.contentLeft}px`;
                });
                container.parentNode.appendChild(resultContainer);
                portal = (0, react_dom_1.createPortal)(this._resolveResultWidget, resultContainer);
                this.resultContainer = resultContainer;
            }
            else {
                throw new Error('[impossible path] container.parentNode is null');
            }
        }
        this.root.render(react_1.default.createElement(ide_core_browser_1.ConfigProvider, { value: this.configContext },
            react_1.default.createElement("div", { className: inline_diff_widget_module_less_1.default.ai_diff_editor_container },
                react_1.default.createElement(DiffContentProvider, { dto: { range: this.range, modifiedValue: this.modifiedValue }, editor: this.editor, onMaxLineCount: (n) => {
                        if (n) {
                            this._relayout(n);
                            this._onMaxLineCount.fire(n);
                        }
                    }, onReady: (handler) => (this.diffWidgetHandler = handler) })),
            portal));
    }
    computeResultWidgetWidth() {
        const layoutInfo = this.editor.getLayoutInfo();
        return layoutInfo.contentWidth + layoutInfo.contentLeft;
    }
    _onWidth(widthInPixel) {
        super._onWidth(widthInPixel);
        requestAnimationFrame(() => {
            if (this.resultContainer) {
                this.resultContainer.style.width = `${this.computeResultWidgetWidth()}px`;
            }
        });
    }
    getModifiedValue() {
        if (this.diffWidgetHandler) {
            return this.diffWidgetHandler.getModifiedValue();
        }
        return this.modifiedValue;
    }
    constructor(id, editor, selection, modifiedValue, hiddenArea) {
        super(editor, {
            showArrow: false,
            showFrame: false,
            arrowColor: undefined,
            frameColor: undefined,
            keepEditorSelection: true,
            showInHiddenAreas: true,
        });
        this.id = id;
        this._onMaxLineCount = new ide_core_browser_1.Emitter();
        this.onMaxLineCount = this._onMaxLineCount.event;
        this.diffWidgetHandler = null;
        this.resultContainer = null;
        this._resolveResultWidget = null;
        this.range = selection;
        this.modifiedValue = modifiedValue;
        this.hiddenArea = hiddenArea || selection;
        editor.setHiddenAreas([this.hiddenArea], this.id);
    }
    // // 覆写 revealRange 函数，使其在 show 的时候编辑器不会定位到对应位置
    revealRange(range, isLastLine) {
        // not implement
    }
    setContainerStyle(style) {
        var _a;
        const keys = Object.keys(style);
        for (const key of keys) {
            if (Object.prototype.hasOwnProperty.call((_a = this.container) === null || _a === void 0 ? void 0 : _a.style, key)) {
                this.container.style[key] = style[key];
            }
        }
    }
    addClassName(type) {
        this.setCssClass(type);
        return this;
    }
    dispose() {
        this.hide();
        super.dispose();
    }
    hide() {
        this.editor.setHiddenAreas([], this.id);
        super.hide();
        if (this.root) {
            this.root.unmount();
        }
    }
    /**
     *
     * @param line 视图区域的起始行号
     * @param heightInLines 视图区域的高度（以行数表示）
     */
    showByLine(line, heightInLines = 20) {
        /**
         * 暂时通过 hack 的方式使其能让 zonewidget 在空白处显示出来
         */
        // @ts-ignore
        this.editor._modelData.viewModel.coordinatesConverter.modelPositionIsVisible = () => true;
        super.show({
            startLineNumber: line,
            startColumn: 1,
            endLineNumber: line,
            endColumn: Number.MAX_SAFE_INTEGER,
        }, heightInLines);
    }
    setResolveResultWidget(widget) {
        this._resolveResultWidget = widget;
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.AppConfig),
    tslib_1.__metadata("design:type", Object)
], InlineDiffWidget.prototype, "configContext", void 0);
InlineDiffWidget = tslib_1.__decorate([
    (0, di_1.Injectable)({ multiple: true }),
    tslib_1.__metadata("design:paramtypes", [String, Object, Object, String, Object])
], InlineDiffWidget);
exports.InlineDiffWidget = InlineDiffWidget;
//# sourceMappingURL=inline-diff-widget.js.map